@@ -1,29 +1,43 @@
 /*
  * FILE:     auddev.c
  * PROGRAM:  RAT
- * AUTHOR:   Orion Hodson 
+ * AUTHOR:   Orion Hodson
  *
- * $Revision: 1.34 $
- * $Date: 1999/05/30 12:39:51 $
- *
- * Copyright (c) 1995-98 University College London
+ * Copyright (c) 1995-2001 University College London
  * All rights reserved.
- *
  */
 
+#ifndef HIDE_SOURCE_STRINGS
+static const char cvsid[] =
+	"$Id: auddev.c,v 1.62 2004/01/12 17:14:18 ucacoxh Exp $";
+#endif /* HIDE_SOURCE_STRINGS */
+
 #include "config_unix.h"
 #include "config_win32.h"
+
 #include "memory.h"
 #include "debug.h"
 #include "audio_types.h"
 #include "audio_fmt.h"
 #include "auddev.h"
+#include "auddev_null.h"
+#include "auddev_luigi.h"
+#include "auddev_newpcm.h"
+#include "auddev_osprey.h"
+#include "auddev_oss.h"
+#include "auddev_ixj.h"
+#include "auddev_alsa.h"
+#include "auddev_pca.h"
+#include "auddev_sgi.h"
+#include "auddev_sparc.h"
+#include "auddev_win32.h"
+#include "auddev_trans.h"
 
 typedef struct {
         int  (*audio_if_init)(void);                 /* Test and initialize audio interface (OPTIONAL)    */
         int  (*audio_if_free)(void);                 /* Free audio interface (OPTIONAL)                   */
         int  (*audio_if_dev_cnt)(void);              /* Device count for interface (REQUIRED)             */
-        const char* 
+        char*
              (*audio_if_dev_name)(int);              /* Device name query (REQUIRED)                      */
 
         int  (*audio_if_open)(int, audio_format *ifmt, audio_format *ofmt); /* Open device with formats   */
@@ -61,20 +75,11 @@ typedef struct {
         int  (*audio_if_format_supported)(int, audio_format *);
 } audio_if_t;
 
-#include "auddev_luigi.h"
-#include "auddev_null.h"
-#include "auddev_osprey.h"
-#include "auddev_oss.h"
-#include "auddev_pca.h"
-#include "auddev_sparc.h"
-#include "auddev_sgi.h"
-#include "auddev_win32.h"
-
 audio_if_t audio_if_table[] = {
-#ifdef IRIX
+#ifdef HAVE_SGI_AUDIO
         {
-                NULL, 
-                NULL, 
+                NULL,
+                NULL,
                 sgi_audio_device_count,
                 sgi_audio_device_name,
                 sgi_audio_open,
@@ -102,8 +107,8 @@ audio_if_t audio_if_table[] = {
                 sgi_audio_wait_for,
                 NULL
         },
-#endif /* IRIX */
-#ifdef Solaris
+#endif /* SGI_AUDIO */
+#ifdef HAVE_SPARC_AUDIO
         {
                 NULL,
                 NULL,
@@ -134,11 +139,11 @@ audio_if_t audio_if_table[] = {
                 sparc_audio_wait_for,
                 sparc_audio_supports
         },
-#endif /* Solaris */
-#ifdef HAVE_OSPREY
+#endif /* HAVE_SPARC_AUDIO */
+#ifdef HAVE_OSPREY_AUDIO
         {
-                osprey_audio_init, 
-                NULL, 
+                osprey_audio_init,
+                NULL,
                 osprey_audio_device_count,
                 osprey_audio_device_name,
                 osprey_audio_open,
@@ -166,10 +171,42 @@ audio_if_t audio_if_table[] = {
                 osprey_audio_wait_for,
                 NULL
         },
-#endif /* HAVE_OSPREY */
-#if defined(Linux)||defined(OSS)
+#endif /* HAVE_OSPREY_AUDIO */
+#ifdef HAVE_ALSA_AUDIO
         {
-                oss_audio_query_devices, 
+                alsa_audio_init,
+                NULL,
+                alsa_get_device_count,
+                alsa_get_device_name,
+                alsa_audio_open,
+                alsa_audio_close,
+                alsa_audio_drain,
+                alsa_audio_duplex,
+                alsa_audio_read,
+                alsa_audio_write,
+                alsa_audio_non_block,
+                alsa_audio_block,
+                alsa_audio_set_igain,
+                alsa_audio_get_igain,
+                alsa_audio_set_ogain,
+                alsa_audio_get_ogain,
+                alsa_audio_loopback,
+                alsa_audio_oport_set,
+                alsa_audio_oport_get,
+                alsa_audio_oport_details,
+                alsa_audio_oport_count,
+                alsa_audio_iport_set,
+                alsa_audio_iport_get,
+                alsa_audio_iport_details,
+                alsa_audio_iport_count,
+                alsa_audio_is_ready,
+                alsa_audio_wait_for,
+                alsa_audio_supports
+        },
+#endif /* HAVE_ALSA_AUDIO */
+#ifdef HAVE_OSS_AUDIO
+        {
+                oss_audio_init,
                 NULL,
                 oss_get_device_count,
                 oss_get_device_name,
@@ -198,13 +235,43 @@ audio_if_t audio_if_table[] = {
                 oss_audio_wait_for,
                 oss_audio_supports
         },
-
-#endif /* Linux / OSS */
-
-#if defined(WIN32)
+#endif /* HAVE_OSS_AUDIO */
+#ifdef HAVE_IXJ_AUDIO
+        {
+                ixj_audio_init,
+                NULL,
+                ixj_get_device_count,
+                ixj_get_device_name,
+                ixj_audio_open,
+                ixj_audio_close,
+                ixj_audio_drain,
+                ixj_audio_duplex,
+                ixj_audio_read,
+                ixj_audio_write,
+                ixj_audio_non_block,
+                ixj_audio_block,
+                ixj_audio_set_igain,
+                ixj_audio_get_igain,
+                ixj_audio_set_ogain,
+                ixj_audio_get_ogain,
+                ixj_audio_loopback,
+                ixj_audio_oport_set,
+                ixj_audio_oport_get,
+                ixj_audio_oport_details,
+                ixj_audio_oport_count,
+                ixj_audio_iport_set,
+                ixj_audio_iport_get,
+                ixj_audio_iport_details,
+                ixj_audio_iport_count,
+                ixj_audio_is_ready,
+                ixj_audio_wait_for,
+                ixj_audio_supports
+        },
+#endif /* HAVE_IXJ_AUDIO */
+#ifdef WIN32
         {
                 w32sdk_audio_init,
-                NULL, 
+                w32sdk_audio_free,
                 w32sdk_get_device_count,
                 w32sdk_get_device_name,
                 w32sdk_audio_open,
@@ -233,8 +300,7 @@ audio_if_t audio_if_table[] = {
                 w32sdk_audio_supports
         },
 #endif /* WIN32 */
-
-#if defined(FreeBSD)
+#ifdef HAVE_LUIGI_AUDIO
         {
                 luigi_audio_query_devices,
                 NULL,
@@ -265,13 +331,43 @@ audio_if_t audio_if_table[] = {
                 luigi_audio_wait_for,
                 luigi_audio_supports
         },
-
-#endif /* FreeBSD */
-
-#if defined(HAVE_PCA)
+#endif /* HAVE_LUIGI_AUDIO */
+#ifdef HAVE_NEWPCM_AUDIO
+        {
+                newpcm_audio_query_devices,
+                NULL,
+                newpcm_get_device_count,
+                newpcm_get_device_name,
+                newpcm_audio_open,
+                newpcm_audio_close,
+                newpcm_audio_drain,
+                newpcm_audio_duplex,
+                newpcm_audio_read,
+                newpcm_audio_write,
+                newpcm_audio_non_block,
+                newpcm_audio_block,
+                newpcm_audio_set_igain,
+                newpcm_audio_get_igain,
+                newpcm_audio_set_ogain,
+                newpcm_audio_get_ogain,
+                newpcm_audio_loopback,
+                newpcm_audio_oport_set,
+                newpcm_audio_oport_get,
+                newpcm_audio_oport_details,
+                newpcm_audio_oport_count,
+                newpcm_audio_iport_set,
+                newpcm_audio_iport_get,
+                newpcm_audio_iport_details,
+                newpcm_audio_iport_count,
+                newpcm_audio_is_ready,
+                newpcm_audio_wait_for,
+                newpcm_audio_supports
+        },
+#endif /* HAVE_NEWPCM_AUDIO */
+#ifdef HAVE_PCA_AUDIO
         {
                 pca_audio_init,
-                NULL, 
+                NULL,
                 pca_audio_device_count,
                 pca_audio_device_name,
                 pca_audio_open,
@@ -299,14 +395,44 @@ audio_if_t audio_if_table[] = {
                 pca_audio_wait_for,
                 pca_audio_supports
         },
-#endif /* HAVE_PCA */
+#endif /* HAVE_PCA_AUDIO */
+	{
+                trans_audio_init,
+                NULL,
+                trans_audio_device_count,
+                trans_audio_device_name,
+                trans_audio_open,
+                trans_audio_close,
+                trans_audio_drain,
+                trans_audio_duplex,
+                trans_audio_read,
+                trans_audio_write,
+                trans_audio_non_block,
+                trans_audio_block,
+                trans_audio_set_igain,
+                trans_audio_get_igain,
+                trans_audio_set_ogain,
+                trans_audio_get_ogain,
+                trans_audio_loopback,
+                trans_audio_oport_set,
+                trans_audio_oport_get,
+                trans_audio_oport_details,
+                trans_audio_oport_count,
+                trans_audio_iport_set,
+                trans_audio_iport_get,
+                trans_audio_iport_details,
+                trans_audio_iport_count,
+                trans_audio_is_ready,
+                trans_audio_wait_for,
+                trans_audio_supports
+	},
         {
                 /* This is the null audio device - it should always go last so that
                  * audio_get_null_device works.  The idea being when we can't get hold
                  * of a real device we fake one.  Prevents lots of problems elsewhere.
                  */
                 NULL,
-                NULL, 
+                NULL,
                 null_audio_device_count,
                 null_audio_device_name,
                 null_audio_open,
@@ -342,7 +468,8 @@ audio_if_t audio_if_table[] = {
  * audio interfaces table.  Audio open returns index to these */
 static audio_desc_t active_device_desc[INITIAL_AUDIO_INTERFACES];
 static int active_devices;
-static int actual_devices, actual_interfaces;
+static uint32_t actual_devices, actual_interfaces;
+static audio_device_details_t *dev_details;
 
 #define MAX_ACTIVE_DEVICES   2
 
@@ -350,7 +477,7 @@ static int actual_devices, actual_interfaces;
 #define AUDDEV_REQ_IFMT      0
 #define AUDDEV_REQ_OFMT      1
 
-/* These are actual device formats that are transparently converted 
+/* These are actual device formats that are transparently converted
  * into the required ones during reads and writes.  */
 #define AUDDEV_ACT_IFMT      2
 #define AUDDEV_ACT_OFMT      3
@@ -361,7 +488,7 @@ static audio_format* fmts[MAX_ACTIVE_DEVICES][AUDDEV_NUM_FORMATS];
 static sample*       convert_buf[MAX_ACTIVE_DEVICES]; /* used if conversions used */
 
 /* Counters for samples read/written */
-static u_int32 samples_read[MAX_ACTIVE_DEVICES], samples_written[MAX_ACTIVE_DEVICES];
+static uint32_t samples_read[MAX_ACTIVE_DEVICES], samples_written[MAX_ACTIVE_DEVICES];
 
 /* We map indexes outside range for file descriptors so people don't attempt
  * to circumvent audio interface.  If something is missing it should be added
@@ -377,41 +504,26 @@ static u_int32 samples_read[MAX_ACTIVE_DEVICES], samples_written[MAX_ACTIVE_DEVI
 
 /*****************************************************************************
  *
- * Code for working out how many devices are, what they are called, and what 
+ * Code for working out how many devices are, what they are called, and what
  * descriptor should be used to access them.
  *
  *****************************************************************************/
 
-int
+uint32_t
 audio_get_device_count()
 {
         return actual_devices;
 }
 
-int
-audio_get_device_details(int idx, audio_device_details_t *add)
+const audio_device_details_t *
+audio_get_device_details(uint32_t idx)
 {
-        int 		 iface = 0;
-	int 		 devs  = 0;
-        const char 	*name  = NULL;
+        assert(idx < actual_devices);
 
-        assert(idx < actual_devices && idx >= 0);
-        assert(add != NULL);
-
-        /* Find interface device number idx belongs to */
-        while((devs = audio_if_table[iface].audio_if_dev_cnt()) && idx >= devs) {
-                iface++;
-                idx -= devs;
+        if (idx < actual_devices) {
+                return &dev_details[idx];
         }
-
-        assert(devs != 0);
-
-        add->descriptor = AIF_MAKE_DESC(iface, idx);
-        assert(audio_if_table[iface].audio_if_dev_name != NULL);
-        name = audio_if_table[iface].audio_if_dev_name(idx);
-        assert(name != NULL);
-        strncpy(add->name, name, AUDIO_DEVICE_NAME_LENGTH);
-        return TRUE;
+        return NULL;
 }
 
 audio_desc_t
@@ -435,13 +547,14 @@ static int
 get_active_device_index(audio_desc_t ad)
 {
         int i;
-        
+
         for (i = 0; i < active_devices; i++) {
                 if (active_device_desc[i] == ad) {
                         return i;
                 }
         }
 
+	debug_msg("Device %d is not active\n", (int) ad);
         return -1;
 }
 
@@ -455,9 +568,10 @@ audio_device_is_open(audio_desc_t ad)
 int
 audio_open(audio_desc_t ad, audio_format *ifmt, audio_format *ofmt)
 {
-        audio_format format;
-        int iface, device, dev_idx;
-        int success;
+        audio_format 	format;
+        int 		iface, device, dev_idx;
+        int 		success;
+	char 		s[50];
 
         assert(AIF_VALID_INTERFACE(ad) && AIF_VALID_DEVICE_NO(ad));
         assert(ifmt != NULL);
@@ -479,7 +593,6 @@ audio_open(audio_desc_t ad, audio_format *ifmt, audio_format *ofmt)
 
         dev_idx   = active_devices;
 
-        assert(get_active_device_index(ad) == -1);
         assert(audio_if_table[iface].audio_if_open);
 
         if (audio_format_get_common(ifmt, ofmt, &format) == FALSE) {
@@ -491,14 +604,15 @@ audio_open(audio_desc_t ad, audio_format *ifmt, audio_format *ofmt)
         fmts[dev_idx][AUDDEV_ACT_OFMT] = audio_format_dup(&format);
 
         /* Formats can get changed in audio_if_open, but only sample
-         * type, not the number of channels or freq 
+         * type, not the number of channels or freq
          */
-        success = audio_if_table[iface].audio_if_open(device, 
-                                                          fmts[dev_idx][AUDDEV_ACT_IFMT], 
+        success = audio_if_table[iface].audio_if_open(device,
+                                                          fmts[dev_idx][AUDDEV_ACT_IFMT],
                                                           fmts[dev_idx][AUDDEV_ACT_OFMT]);
 
         if (success) {
                 /* Add device to list of those active */
+                debug_msg("Opened device: %s\n", audio_if_table[iface].audio_if_dev_name(device));
                 active_device_desc[dev_idx] = ad;
                 active_devices ++;
 
@@ -517,34 +631,30 @@ audio_open(audio_desc_t ad, audio_format *ifmt, audio_format *ofmt)
                         audio_close(ad);
                         return FALSE;
                 }
-                
-                /* If we are going to need conversion between requested and 
+
+                /* If we are going to need conversion between requested and
                  * actual device formats store requested formats */
                 if (!audio_format_match(ifmt, fmts[dev_idx][AUDDEV_ACT_IFMT])) {
                         fmts[dev_idx][AUDDEV_REQ_IFMT] = audio_format_dup(ifmt);
-#ifdef DEBUG
-                        {
-                                char s[50];
-                                audio_format_name(fmts[dev_idx][AUDDEV_REQ_IFMT], s, 50);
-                                debug_msg("Requested Input: %s\n", s);
-                                audio_format_name(fmts[dev_idx][AUDDEV_ACT_IFMT], s, 50);
-                                debug_msg("Actual Input:    %s\n", s);
-                        }
-#endif
-                }
+			audio_format_name(fmts[dev_idx][AUDDEV_REQ_IFMT], s, 50);
+			debug_msg("Requested Input: %s\n", s);
+			audio_format_name(fmts[dev_idx][AUDDEV_ACT_IFMT], s, 50);
+			debug_msg("Actual Input:    %s\n", s);
+                } else {
+			audio_format_name(fmts[dev_idx][AUDDEV_ACT_IFMT], s, 50);
+			debug_msg("Input:  %s\n", s);
+		}
 
                 if (!audio_format_match(ofmt, fmts[dev_idx][AUDDEV_ACT_OFMT])) {
                         fmts[dev_idx][AUDDEV_REQ_OFMT] = audio_format_dup(ofmt);
-#ifdef DEBUG
-                        {
-                                char s[50];
-                                audio_format_name(fmts[dev_idx][AUDDEV_REQ_OFMT], s, 50);
-                                debug_msg("Requested Output: %s\n", s);
-                                audio_format_name(fmts[dev_idx][AUDDEV_ACT_OFMT], s, 50);
-                                debug_msg("Actual Output:    %s\n", s);
-                        }
-#endif
-                }
+			audio_format_name(fmts[dev_idx][AUDDEV_REQ_OFMT], s, 50);
+			debug_msg("Requested Output: %s\n", s);
+			audio_format_name(fmts[dev_idx][AUDDEV_ACT_OFMT], s, 50);
+			debug_msg("Actual Output:    %s\n", s);
+                } else {
+			audio_format_name(fmts[dev_idx][AUDDEV_ACT_OFMT], s, 50);
+			debug_msg("Output: %s\n", s);
+		}
 
                 if (fmts[dev_idx][AUDDEV_REQ_IFMT] || fmts[dev_idx][AUDDEV_REQ_OFMT]) {
                         convert_buf[dev_idx] = (sample*)xmalloc(DEVICE_REC_BUF); /* is this in samples or bytes ? */
@@ -582,9 +692,12 @@ audio_close(audio_desc_t ad)
         for(i = 0; i < active_devices; i++) {
                 if (active_device_desc[i] == ad) {
                         for(k = 0; k < AUDDEV_NUM_FORMATS; k++) {
-                                if (fmts[i][k] != NULL) audio_format_free(&fmts[i][k]);                                
+                                if (fmts[i][k] != NULL) audio_format_free(&fmts[i][k]);
+                        }
+                        if (convert_buf[i]) {
+                                xfree(convert_buf[i]);
+                                convert_buf[i] = NULL;
                         }
-                        if (convert_buf[i]) xfree(convert_buf[i]);
                         samples_written[i] = 0;
                         samples_read[i]    = 0;
                 } else {
@@ -642,10 +755,10 @@ audio_drain(audio_desc_t ad)
 
         assert(AIF_VALID_INTERFACE(ad) && AIF_VALID_DEVICE_NO(ad));
         assert(audio_device_is_open(ad));
-        
+
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         audio_if_table[iface].audio_if_drain(device);
 }
 
@@ -672,15 +785,16 @@ audio_read(audio_desc_t ad, sample *buf, int samples)
         int device, iface;
         int idx = get_active_device_index(ad);
 
-        assert(AIF_VALID_INTERFACE(ad) && AIF_VALID_DEVICE_NO(ad));        
+        assert(AIF_VALID_INTERFACE(ad) && AIF_VALID_DEVICE_NO(ad));
         assert(idx >= 0 && idx < active_devices);
         assert(buf != NULL);
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
 
+/*
         xmemchk();
-
+        */
         if (fmts[idx][AUDDEV_REQ_IFMT] == NULL) {
                 /* No conversion necessary as input format and real format are
                  * the same. [Input format only allocated if different from
@@ -699,7 +813,7 @@ audio_read(audio_desc_t ad, sample *buf, int samples)
                 samples_read[idx] += read_len / (sample_size * fmts[idx][AUDDEV_REQ_IFMT]->channels);
         }
 
-        xmemchk();
+/*        xmemchk(); */
         return read_len / sample_size;
 }
 
@@ -709,13 +823,13 @@ audio_write(audio_desc_t ad, sample *buf, int len)
         int write_len ,sample_size;
         int iface, device;
         int idx = get_active_device_index(ad);
-        
+
         assert(idx >= 0 && idx < active_devices);
         assert(AIF_VALID_INTERFACE(ad) && AIF_VALID_DEVICE_NO(ad));
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         xmemchk();
 
         if (fmts[idx][AUDDEV_REQ_OFMT] == NULL) {
@@ -766,7 +880,7 @@ audio_block(audio_desc_t ad)
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         audio_if_table[iface].audio_if_block(device);
 }
 
@@ -800,6 +914,7 @@ audio_get_igain(audio_desc_t ad)
         device = AIF_GET_DEVICE_NO(ad);
 
         gain = audio_if_table[iface].audio_if_get_igain(device);
+	debug_msg("GAIN=%d\n", gain);
 
         assert(gain >= 0);
         assert(gain <= MAX_AMP);
@@ -836,7 +951,7 @@ audio_get_ogain(audio_desc_t ad)
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         volume = audio_if_table[iface].audio_if_get_ogain(device);
         assert(volume >= 0);
         assert(volume <= MAX_AMP);
@@ -858,7 +973,9 @@ audio_loopback(audio_desc_t ad, int gain)
         assert(gain >= 0);
         assert(gain <= MAX_AMP);
 
-        if (audio_if_table[iface].audio_if_loopback) audio_if_table[iface].audio_if_loopback(device, gain);
+        if (audio_if_table[iface].audio_if_loopback) {
+                audio_if_table[iface].audio_if_loopback(device, gain);
+        }
 }
 
 void
@@ -871,7 +988,7 @@ audio_set_oport(audio_desc_t ad, audio_port_t port)
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         audio_if_table[iface].audio_if_set_oport(device, port);
 }
 
@@ -913,7 +1030,7 @@ audio_get_oport_count(audio_desc_t ad)
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         return audio_if_table[iface].audio_if_get_oport_count(device);
 }
 
@@ -969,7 +1086,7 @@ audio_get_iport_count(audio_desc_t ad)
 
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
-        
+
         return audio_if_table[iface].audio_if_get_iport_count(device);
 }
 
@@ -1004,7 +1121,7 @@ audio_wait_for(audio_desc_t ad, int delay_ms)
 /* Code for adding/initialising/removing audio ifaces */
 
 int
-audio_device_supports(audio_desc_t ad, u_int16 rate, u_int16 channels)
+audio_device_supports(audio_desc_t ad, uint16_t rate, uint16_t channels)
 {
         int iface, device;
 
@@ -1014,7 +1131,7 @@ audio_device_supports(audio_desc_t ad, u_int16 rate, u_int16 channels)
         iface  = AIF_GET_INTERFACE(ad);
         device = AIF_GET_DEVICE_NO(ad);
 
-        if (rate % 8000 || channels > 2) {
+        if (((rate % 8000) && (rate % 11025)) || channels > 2) {
                 debug_msg("Invalid combo %d Hz %d channels\n", rate, channels);
                 return FALSE;
         }
@@ -1032,28 +1149,28 @@ audio_device_supports(audio_desc_t ad, u_int16 rate, u_int16 channels)
         return TRUE;
 }
 
-u_int32
+uint32_t
 audio_get_device_time(audio_desc_t ad)
 {
         audio_format *fmt;
-        u_int32       samples_per_block;
+        uint32_t       samples_per_block;
         int dev = get_active_device_index(ad);
 
         assert(AIF_VALID_INTERFACE(ad) && AIF_VALID_DEVICE_NO(ad));
         assert(dev >= 0 && dev < active_devices);
 
         if (fmts[dev][AUDDEV_REQ_IFMT]) {
-                fmt = fmts[dev][AUDDEV_REQ_IFMT]; 
+                fmt = fmts[dev][AUDDEV_REQ_IFMT];
         } else {
-                fmt = fmts[dev][AUDDEV_ACT_IFMT]; 
+                fmt = fmts[dev][AUDDEV_ACT_IFMT];
         }
-        
+
         samples_per_block = fmt->bytes_per_block * 8 / (fmt->channels * fmt->bits_per_sample);
-        
+
         return (samples_read[dev]/samples_per_block) * samples_per_block;
 }
 
-u_int32
+uint32_t
 audio_get_samples_read(audio_desc_t ad)
 {
         int dev = get_active_device_index(ad);
@@ -1062,7 +1179,7 @@ audio_get_samples_read(audio_desc_t ad)
         return samples_read[dev];
 }
 
-u_int32
+uint32_t
 audio_get_samples_written(audio_desc_t ad)
 {
         int dev = get_active_device_index(ad);
@@ -1074,13 +1191,19 @@ audio_get_samples_written(audio_desc_t ad)
 int
 audio_init_interfaces(void)
 {
-        u_int32 i, j, n, devs[INITIAL_AUDIO_INTERFACES];
+        static int inited = 0;
+        uint32_t i, j, k, n, devs[INITIAL_AUDIO_INTERFACES];
+
+        if (inited) {
+                return 0;
+        }
+        inited++;
 
         actual_devices = 0;
         actual_interfaces = INITIAL_AUDIO_INTERFACES;
         for(i = 0; i < INITIAL_AUDIO_INTERFACES; i++) {
                 if (audio_if_table[i].audio_if_init) {
-                        audio_if_table[i].audio_if_init(); 
+                        audio_if_table[i].audio_if_init();
                 }
                 assert(audio_if_table[i].audio_if_dev_cnt);
 		devs[i] = audio_if_table[i].audio_if_dev_cnt();
@@ -1101,20 +1224,45 @@ audio_init_interfaces(void)
                 }
         }
 
+        /* Create device details table and fill it in */
+        dev_details = (audio_device_details_t*)xmalloc(sizeof(audio_device_details_t) * actual_devices);
+        k = 0;
+        for (i = 0; i < actual_interfaces; i++) {
+                n = audio_if_table[i].audio_if_dev_cnt();
+                assert(n > 0);
+                for (j = 0; j < n; j++) {
+                        dev_details[k].name       = xstrdup(audio_if_table[i].audio_if_dev_name(j));
+                        dev_details[k].descriptor = AIF_MAKE_DESC(i, j);
+			debug_msg("Added \"%s\" to audio device table\n", dev_details[k].name);
+                        k++;
+                }
+        }
+        assert(k == actual_devices);
+
         return TRUE;
 }
 
 int
 audio_free_interfaces(void)
 {
-        u_int32 i;
+        uint32_t i;
 
+	debug_msg("audio_free_interfaces %d\n", INITIAL_AUDIO_INTERFACES);
         for(i = 0; i < INITIAL_AUDIO_INTERFACES; i++) {
+		debug_msg("%p\n", audio_if_table[i].audio_if_free);
                 if (audio_if_table[i].audio_if_free) {
-                        audio_if_table[i].audio_if_free(); 
+			debug_msg("Freeing audio interface %d\n", i);
+                        audio_if_table[i].audio_if_free();
                 }
         }
 
+        if (dev_details != NULL) {
+                for(i = 0; i < actual_devices; i++) {
+                        xfree(dev_details[i].name);
+                }
+                xfree(dev_details);
+                dev_details = NULL;
+        }
+
         return TRUE;
 }
-
