@@ -3,12 +3,10 @@
  * PROGRAM: RAT
  * AUTHOR:  Isidor Kouvelas / Orion Hodson / Colin Perkins
  *
- * $Revision: 1.3 $
- * $Date: 1999/03/24 23:01:22 $
- *
- * Copyright (c) 1995-98 University College London
+ * Copyright (c) 1995-2001 University College London
  * All rights reserved.
  *
+ * $Id: audio_util.h,v 1.13 2004/01/12 17:14:24 ucacoxh Exp $
  */
 
 #ifndef __AUDIO_UTIL_H__
@@ -23,6 +21,15 @@ BOOL    mmx_present();
 void    audio_mix_mmx(sample *dst, sample *in, int len);
 #endif
 
+void    audio_scale_buffer(sample *buf, int len, double scale);
+
+/* Energy calculation */
+uint16_t audio_avg_energy (sample *buf, uint32_t dur, uint32_t channels);
+
+sample   audio_abs_max(sample *buf, uint32_t samples);
+
+void     audio_blend(sample *from, sample *to, sample *dst, int samples, int channels);
+
 /* Biasing operations */
 
 struct s_bias_ctl;
