@@ -1,18 +1,20 @@
 /*
  * FILE:      cc_rdncy.c
- * AUTHOR(S): Orion Hodson 
- *	
- * $Revision: 1.28 $
- * $Date: 1999/05/13 07:01:57 $
- * 
- * Copyright (c) 1995-99 University College London
- * All rights reserved.
+ * AUTHOR(S): Orion Hodson
  *
+ *
+ * Copyright (c) 1995-2001 University College London
+ * All rights reserved.
  */
 
+#ifndef HIDE_SOURCE_STRINGS
+static const char cvsid[] =
+	"$Id: cc_rdncy.c,v 1.43 2004/01/09 18:35:40 ucacoxh Exp $";
+#endif /* HIDE_SOURCE_STRINGS */
+
 #include "config_unix.h"
 #include "config_win32.h"
-
+#include "audio_types.h"
 #include "codec_types.h"
 #include "codec.h"
 #include "channel_types.h"
@@ -23,8 +25,6 @@
 #include "util.h"
 #include "debug.h"
 
-#include "timers.h"
-
 #define RED_MAX_LAYERS 3
 #define RED_MAX_OFFSET 16383u
 #define RED_MAX_LEN    1023u
@@ -47,22 +47,22 @@
 #define RED_HDR8_GET_PT(z)     (z)
 
 typedef struct s_red_layer {
-        codec_id_t          cid;
-        u_int32             pkts_off;
+        codec_id_t cid;
+        uint32_t   pkts_off;
 } red_layer;
 
 typedef struct {
-        red_layer            layer[RED_MAX_LAYERS];
-        u_int32              n_layers;
+        red_layer             layer[RED_MAX_LAYERS];
+        uint32_t              n_layers;
         struct s_pb          *media_buffer;
         struct s_pb_iterator *media_pos;
-        u_int32               units_ready;
-        ts_t                  history; /* How much audio history is needed for coding */
-        ts_t                  last_in; /* timestamp of last media unit accepted */
+        uint32_t              units_ready;
+        timestamp_t           history; /* How much audio history is needed for coding */
+        timestamp_t           last_in; /* timestamp of last media unit accepted */
 } red_enc_state;
 
 int
-redundancy_encoder_create(u_char **state, u_int32 *len)
+redundancy_encoder_create(u_char **state, uint32_t *len)
 {
         red_enc_state *re = (red_enc_state*)xmalloc(sizeof(red_enc_state));
 
@@ -86,7 +86,7 @@ redundancy_encoder_create(u_char **state, u_int32 *len)
                 goto fail_pb_iterator;
         }
 
-        re->n_layers    = 0;        
+        re->n_layers    = 0;
         re->units_ready = 0;
 
         return TRUE;
@@ -103,10 +103,10 @@ fail_alloc:
 }
 
 void
-redundancy_encoder_destroy(u_char **state, u_int32 len)
+redundancy_encoder_destroy(u_char **state, uint32_t len)
 {
         red_enc_state *re = *((red_enc_state**)state);
-        
+
         assert(len == sizeof(red_enc_state));
 
         pb_iterator_destroy(re->media_buffer,
@@ -122,7 +122,7 @@ int
 redundancy_encoder_reset(u_char *state)
 {
         red_enc_state *re = (red_enc_state*)state;
-        
+
         pb_flush(re->media_buffer);
         re->units_ready = 0;
 
@@ -131,9 +131,9 @@ redundancy_encoder_reset(u_char *state)
 
 /* Adds header to next free slot in channel_data */
 static void
-add_hdr(channel_unit *chu, int hdr_type, codec_id_t cid, u_int32 uo, u_int32 len)
+add_hdr(channel_unit *chu, int hdr_type, codec_id_t cid, uint32_t uo, uint32_t len)
 {
-        u_int32 so;             /* sample time offset */
+        uint32_t so;             /* sample time offset */
         u_char  pt;
 
         assert(chu != NULL);
@@ -148,10 +148,10 @@ add_hdr(channel_unit *chu, int hdr_type, codec_id_t cid, u_int32 uo, u_int32 len
         assert(len <= RED_MAX_LEN );
 
         if (hdr_type == RED_EXTRA) {
-                u_int32 *h;
-                h = (u_int32*)block_alloc(4);
+                uint32_t *h;
+                h = (uint32_t*)block_alloc(4);
                 RED_HDR32_INIT(*h);
-                RED_HDR32_SET_PT(*h, (u_int32)pt);
+                RED_HDR32_SET_PT(*h, (uint32_t)pt);
                 RED_HDR32_SET_OFF(*h, so);
                 RED_HDR32_SET_LEN(*h, len);
                 *h = htonl(*h);
@@ -170,23 +170,25 @@ add_hdr(channel_unit *chu, int hdr_type, codec_id_t cid, u_int32 uo, u_int32 len
 
 /* make_pdu - converts a string of coded units into a channel_unit */
 
-static u_int32
+static uint32_t
 make_pdu(struct s_pb_iterator *pbi,
-         u_int32               upp,
+         uint32_t               upp,
          codec_id_t            cid,
          channel_data         *out)
 {
         struct s_pb_iterator *p;
-        u_int32        i, j, md_len, used;
+        uint32_t        i, j, md_len, used;
+        u_char        *md_get;
         media_data    *md;
-        ts_t           playout;
+        timestamp_t           playout;
         int            success;
 
         pb_iterator_dup(&p, pbi);
 
         used = 0;
         for (i = 0; i < upp; i++) {
-                success = pb_iterator_get_at(p, (u_char**)&md, &md_len, &playout);
+                success = pb_iterator_get_at(p, &md_get, &md_len, &playout);
+                md = (media_data*)md_get;
                 assert(success); /* We could rewind this far so must be able to get something! */
 
                 /* Find first compatible coding */
@@ -215,7 +217,7 @@ make_pdu(struct s_pb_iterator *pbi,
                 md->rep[j]->id       = 0; /* nobble this unit since we have taken it's data */
                 used++;
                 assert(used <= out->nelem);
-                
+
                 pb_iterator_advance(p);
         }
 
@@ -226,11 +228,11 @@ make_pdu(struct s_pb_iterator *pbi,
 }
 
 static channel_data *
-redundancy_encoder_output(red_enc_state *re, u_int32 upp)
+redundancy_encoder_output(red_enc_state *re, uint32_t upp)
 {
         struct s_pb_iterator *pbm;
         channel_data         *cd_coded[RED_MAX_LAYERS], *cd_out;
-        u_int32               offset ;
+        uint32_t               offset ;
         int                   i, j, layers, success = 0,  used = 0;
 
         pbm = re->media_pos;
@@ -238,15 +240,15 @@ redundancy_encoder_output(red_enc_state *re, u_int32 upp)
 
         /*** Stage 1: Packing coded audio units ******************************/
 
-        /* Rewind iterator to start of first pdu */ 
-        for(i = 1; (u_int32)i < upp; i++) {
+        /* Rewind iterator to start of first pdu */
+        for(i = 1; (uint32_t)i < upp; i++) {
                 success = pb_iterator_retreat(pbm);
                 assert(success);
         }
 
         offset = 0;
         layers = 0;
-        for (i = 0; (u_int32)i < re->n_layers; i++) {
+        for (i = 0; (uint32_t)i < re->n_layers; i++) {
                 if (re->units_ready <= re->layer[i].pkts_off * upp) {
                         break;
                 }
@@ -258,7 +260,7 @@ redundancy_encoder_output(red_enc_state *re, u_int32 upp)
                 }
                 xmemchk();
                 /* need upp data elements + 1 for state */
-                channel_data_create(&cd_coded[i], upp + 1); 
+                channel_data_create(&cd_coded[i], upp + 1);
                 success = make_pdu(pbm, upp, re->layer[i].cid, cd_coded[i]);
                 /* make_pdu may fail because coding not available */
                 if (success == FALSE) {
@@ -278,11 +280,11 @@ redundancy_encoder_output(red_enc_state *re, u_int32 upp)
 
         /*** Stage 2: Packing redundancy headers *****************************/
         used = 0;
-        if ((u_int32)layers != re->n_layers) {
+        if ((uint32_t)layers != re->n_layers) {
                 /* Add max offset if we didn't make all units */
-                add_hdr(cd_out->elem[used], 
-                        RED_EXTRA, 
-                        re->layer[re->n_layers - 1].cid, 
+                add_hdr(cd_out->elem[used],
+                        RED_EXTRA,
+                        re->layer[re->n_layers - 1].cid,
                         re->layer[re->n_layers - 1].pkts_off * upp,
                         0);
                 used++;
@@ -290,16 +292,16 @@ redundancy_encoder_output(red_enc_state *re, u_int32 upp)
 
         i = layers - 1;
         while (i > 0) {
-                add_hdr(cd_out->elem[used], 
-                        RED_EXTRA, 
-                        re->layer[re->n_layers - 1].cid, 
+                add_hdr(cd_out->elem[used],
+                        RED_EXTRA,
+                        re->layer[re->n_layers - 1].cid,
                         re->layer[re->n_layers - 1].pkts_off * upp,
                         channel_data_bytes(cd_coded[i]));
                 used++;
                 i--;
         }
 
-        add_hdr(cd_out->elem[used], 
+        add_hdr(cd_out->elem[used],
                 RED_PRIMARY,
                 re->layer[0].cid,
                 re->layer[0].pkts_off * upp,
@@ -330,11 +332,12 @@ int
 redundancy_encoder_encode (u_char      *state,
                            struct s_pb *in,
                            struct s_pb *out,
-                           u_int32      upp)
+                           uint32_t      upp)
 {
-        u_int32        m_len;
-        ts_t           playout;
+        uint32_t        m_len;
+        timestamp_t           playout;
         struct s_pb_iterator *pi;
+        u_char         *m_get;
         media_data     *m;
         red_enc_state  *re = (red_enc_state*)state;
 
@@ -344,7 +347,8 @@ redundancy_encoder_encode (u_char      *state,
 
         assert(pi != NULL);
         pb_iterator_advance(pi);
-        while(pb_iterator_detach_at(pi, (u_char**)&m, &m_len, &playout)) {
+        while(pb_iterator_detach_at(pi, &m_get, &m_len, &playout)) {
+                m = (media_data*)m_get;
                 /* Remove element from playout buffer - it belongs to
                  * the redundancy encoder now.  */
 #ifdef DEBUG_REDUNDANCY
@@ -361,7 +365,7 @@ redundancy_encoder_encode (u_char      *state,
                 re->last_in = playout;
 
                 if (m->nrep > 0) {
-                        pb_add(re->media_buffer, 
+                        pb_add(re->media_buffer,
                                (u_char*)m,
                                m_len,
                                playout);
@@ -385,7 +389,7 @@ redundancy_encoder_encode (u_char      *state,
                         cd = redundancy_encoder_output(re, upp);
                         assert(cd != NULL);
                         s  = pb_add(out, (u_char*)cd, sizeof(channel_data), playout);
-#ifdef DEBUG_REDUNDANCY 
+#ifdef DEBUG_REDUNDANCY
                         debug_msg("Ready %d, Added %d\n", re->units_ready, playout.ticks);
 #endif /* DEBUG_REDUNDANCY */
                         assert(s);
@@ -400,14 +404,15 @@ redundancy_encoder_encode (u_char      *state,
 /* Redundancy {get,set} parameters expects strings like:
  *            dvi-8k-mono/0/lpc-8k-mono/2
  * where number is the offset in number of units.
- */ 
+ */
 
 int
 redundancy_encoder_set_parameters(u_char *state, char *cmd)
 {
+        u_char *encbuf;
         red_enc_state *n, *cur;
         const codec_format_t *cf;
-        u_int32 nl, po;
+        uint32_t nl, po;
         codec_id_t  cid;
         char *s;
         int success = FALSE;
@@ -416,7 +421,8 @@ redundancy_encoder_set_parameters(u_char *state, char *cmd)
         assert(cmd   != NULL);
 
         /* Create a temporary encoder, try to set it's params */
-        redundancy_encoder_create((u_char**)&n, &nl);
+        redundancy_encoder_create(&encbuf, &nl);
+        n = (red_enc_state*)encbuf;
         assert(n != NULL);
 
         s = (char *) strtok(cmd, "/");
@@ -433,7 +439,7 @@ redundancy_encoder_set_parameters(u_char *state, char *cmd)
                 debug_msg("offset too big\n");
                 goto done;
         }
-        
+
         n->layer[0].cid       = cid;
         n->layer[0].pkts_off  = po;
         n->n_layers           = 1;
@@ -457,7 +463,7 @@ redundancy_encoder_set_parameters(u_char *state, char *cmd)
                         debug_msg("offset too big\n");
                         goto done;
                 }
-        
+
                 n->layer[n->n_layers].cid      = cid;
                 n->layer[n->n_layers].pkts_off = po;
                 n->n_layers ++;
@@ -473,21 +479,22 @@ redundancy_encoder_set_parameters(u_char *state, char *cmd)
         /* work out history = duration of audio frame * maximum offset */
         cf = codec_get_format(cur->layer[cur->n_layers - 1].cid);
         cur->history = ts_map32(cf->format.sample_rate,
-                                codec_get_samples_per_frame(cur->layer[cur->n_layers - 1].cid) * 
+                                codec_get_samples_per_frame(cur->layer[cur->n_layers - 1].cid) *
                                 cur->layer[cur->n_layers - 1].pkts_off);
 
         success = TRUE;
 done:
-        redundancy_encoder_destroy((u_char**)&n, nl);
+        encbuf = (u_char*)n;
+        redundancy_encoder_destroy(&encbuf, nl);
         return success;
 }
 
-int 
-redundancy_encoder_get_parameters(u_char *state, char *buf, u_int32 blen)
+int
+redundancy_encoder_get_parameters(u_char *state, char *buf, uint32_t blen)
 {
         const codec_format_t *cf;
         red_enc_state *r;
-        u_int32 i, used, flen;
+        uint32_t i, used, flen;
 
         char frag[CODEC_LONG_NAME_LEN+5]; /* XXX/nn/\0 + 1*/
 
@@ -499,7 +506,7 @@ redundancy_encoder_get_parameters(u_char *state, char *buf, u_int32 blen)
                 debug_msg("Redundancy encoder has not had parameters set!\n");
                 return FALSE;
         }
-        
+
         *buf = '\0';
 	flen = 0;
 
@@ -507,7 +514,7 @@ redundancy_encoder_get_parameters(u_char *state, char *buf, u_int32 blen)
                 cf = codec_get_format(r->layer[i].cid);
                 assert(cf != NULL);
                 sprintf(frag,
-                        "%s/%ld/",
+                        "%s/%d/",
                         cf->long_name,
                         r->layer[i].pkts_off);
                 flen += strlen(frag);
@@ -525,17 +532,17 @@ redundancy_encoder_get_parameters(u_char *state, char *buf, u_int32 blen)
 }
 
 int
-redundancy_decoder_peek(u_int8   pkt_pt,
+redundancy_decoder_peek(uint8_t   pkt_pt,
                         u_char  *buf,
-                        u_int32  len,
-                        u_int16  *upp,
-                        u_int8   *pt)
+                        uint32_t  len,
+                        uint16_t  *upp,
+                        uint8_t   *pt)
 {
         const codec_format_t *cf;
         codec_id_t            cid;
         u_char               *p, *data;
-        u_int32               hdr32, dlen, blen;
-        u_int16               units;        
+        uint32_t               hdr32, dlen, blen;
+        uint16_t               units;
         assert(buf != NULL);
         assert(upp != NULL);
         assert(pt  != NULL);
@@ -544,27 +551,27 @@ redundancy_decoder_peek(u_int8   pkt_pt,
          * advance data pointer past them.
          */
         p = data = buf;
-        hdr32 = ntohl(*(u_int32*)p);
+        hdr32 = ntohl(*(uint32_t*)p);
         while ((hdr32 & RED_HDR32_PAT)) {
                 blen = RED_HDR32_GET_LEN(hdr32);
                 p    += 4; /* goto next hdr */
                 data += 4 + blen;
-                hdr32 = ntohl(*(u_int32*)p);
-                assert(((u_int32)data - (u_int32)buf) <= len);
+                hdr32 = ntohl(*(uint32_t*)p);
+                assert(((unsigned long)data - (unsigned long)buf) <= len);
         }
 
         *pt = *p;
         data += 1; /* step over payload field of primary */
 
         cid = codec_get_by_payload(*pt);
-        
+
         if (!cid) {
                 debug_msg("Codec not found\n");
                 return FALSE;
         }
 
         /* Primary data length */
-        dlen = len - (u_int32)(data - buf);
+        dlen = len - (uint32_t)(data - buf);
 
         cf = codec_get_format(cid);
         assert(cf);
@@ -572,16 +579,16 @@ redundancy_decoder_peek(u_int8   pkt_pt,
         data += cf->mean_per_packet_state_size;
         dlen -= cf->mean_per_packet_state_size;
 
-        assert(((u_int32)data - (u_int32)buf) <= len);
+        assert(((unsigned long)data - (unsigned long)buf) <= len);
 
-        units = 0;        
+        units = 0;
         while (dlen != 0) {
-                blen = codec_peek_frame_size(cid, p, (u_int16)dlen);
+                blen = codec_peek_frame_size(cid, p, (uint16_t)dlen);
                 assert(blen != 0);
                 data += blen;
                 dlen -= blen;
                 units ++;
-                assert(((u_int32)data - (u_int32)buf) <= len);
+                assert(((unsigned long)data - (unsigned long)buf) <= len);
         }
 
         *upp = units;
@@ -599,24 +606,24 @@ redundancy_decoder_peek(u_int8   pkt_pt,
  */
 
 int
-redundancy_decoder_describe (u_int8   pkt_pt,
+redundancy_decoder_describe (uint8_t   pkt_pt,
                              u_char  *data,
-                             u_int32  data_len,
+                             uint32_t  data_len,
                              char    *out,
-                             u_int32  out_len)
+                             uint32_t  out_len)
 {
         const codec_format_t *cf;
         codec_id_t            cid;
-        u_int32 hdr32, slen, blksz, off, nlen;
+        uint32_t hdr32, slen, blksz, off, nlen;
         u_char  *p, pt;
 
         UNUSED(pkt_pt);
-        
+
         *out = '\0';
         slen = 0;
 
         p   = data;
-        hdr32 = ntohl(*((u_int32*)p));
+        hdr32 = ntohl(*((uint32_t*)p));
         while (hdr32 & RED_HDR32_PAT) {
                 pt    = (u_char)RED_HDR32_GET_PT(hdr32);
                 off   = RED_HDR32_GET_OFF(hdr32);
@@ -625,13 +632,13 @@ redundancy_decoder_describe (u_int8   pkt_pt,
 
                 if (cid == 0) {
                         p += 4;
-                        hdr32 = ntohl(*((u_int32*)p));
+                        hdr32 = ntohl(*((uint32_t*)p));
                         continue;
                 }
 
                 cf = codec_get_format(cid);
                 assert(cf != NULL);
-                
+
                 nlen = strlen(cf->long_name);
 
                 if (slen + nlen >=  out_len) {
@@ -647,8 +654,8 @@ redundancy_decoder_describe (u_int8   pkt_pt,
                 slen++;
                 out[nlen+1] = '\0';
                 p += 4;
-                assert((u_int32)(p - data) < data_len);
-                hdr32 = ntohl(*((u_int32*)p));
+                assert((uint32_t)(p - data) < data_len);
+                hdr32 = ntohl(*((uint32_t*)p));
         }
 
         pt  = *p;
@@ -659,9 +666,9 @@ redundancy_decoder_describe (u_int8   pkt_pt,
 
         cf = codec_get_format(cid);
         assert(cf != NULL);
-                
+
         nlen = strlen(cf->long_name);
-        
+
         if (slen + nlen >=  out_len) {
                 debug_msg("Out of buffer space\n");
                 return FALSE;
@@ -674,35 +681,57 @@ redundancy_decoder_describe (u_int8   pkt_pt,
         /* Axe trailing separator */
         out[slen-1] = '\0';
 
-        return TRUE;        
+        return TRUE;
 }
 
-__inline static void
+static int
 place_unit(media_data *md, coded_unit *cu)
 {
+        int16_t i;
 #ifdef DEBUG_REDUNDANCY
         const codec_format_t *cf;
         cf = codec_get_format(cu->id);
         debug_msg("%d %s\n", md->nrep, cf->long_name);
 #endif /* DEBUG_REDUNDANCY */
         assert(md->nrep < MAX_MEDIA_UNITS);
+
+        for (i = 0; i < md->nrep; i++) {
+                if (md->rep[i]->id == cu->id) {
+                        return FALSE;
+                }
+        }
+
+        if (md->nrep > 0 && codec_is_native_coding(md->rep[md->nrep - 1]->id)) {
+                /* Buffer shifts can mean redundancy is received after primary decoded.  */
+                /* Just discard. i.e. pkt 1 (t1 t0) pkt2 (t2 t1), if pkt2 arrives after  */
+                /* pkt1 decoded we don't want to append redundant t1 data as it confuses */
+                /* decoder.                                                              */
+                /* XXX Should check for packet re-ordering i.e. pkt2 arrives and is      */
+                /* decoded before pkt 1 then should use pkt 1's data as this will be     */
+                /* higher quality under normal circumstances.                            */
+                return FALSE;
+        }
+
         md->rep[md->nrep] = cu;
         md->nrep++;
+        return TRUE;
 }
 
 static media_data *
-red_media_data_create_or_get(struct s_pb *p, ts_t playout)
+red_media_data_create_or_get(struct s_pb *p, timestamp_t playout)
 {
         struct s_pb_iterator *pi;
-        media_data *md;
-        u_int32     md_len, success;
-        ts_t        md_playout;
+        u_char               *md_get;
+        media_data           *md;
+        uint32_t              md_len, success;
+        timestamp_t           md_playout;
 
         pb_iterator_create(p, &pi);
         /* iterator is attached to sentinel - can move back or forwards */
 
         while(pb_iterator_retreat(pi)) {
-                success = pb_iterator_get_at(pi, (u_char**)&md, &md_len, &md_playout);
+                success = pb_iterator_get_at(pi, &md_get, &md_len, &md_playout);
+                md = (media_data*)md_get;
                 assert(success);
                 if (ts_eq(md_playout, playout)) {
                         goto done;
@@ -722,12 +751,12 @@ done:
         return md;
 }
 
-__inline static void
+static void
 red_split_unit(u_char  ppt,        /* Primary payload type */
                u_char  bpt,        /* Block payload type   */
                u_char *b,          /* Block pointer        */
-               u_int32 blen,       /* Block len            */
-               ts_t    playout,    /* Block playout time   */
+               uint32_t blen,       /* Block len            */
+               timestamp_t    playout,    /* Block playout time   */
                struct s_pb *out)   /* media buffer         */
 {
         const codec_format_t *cf;
@@ -735,7 +764,7 @@ red_split_unit(u_char  ppt,        /* Primary payload type */
         codec_id_t  cid, pid;
         coded_unit *cu;
         u_char     *p,*pe;
-        ts_t        step;
+        timestamp_t        step;
 
         pid = codec_get_by_payload(ppt);
         if (!pid) {
@@ -748,7 +777,7 @@ red_split_unit(u_char  ppt,        /* Primary payload type */
                 debug_msg("Payload not recognized\n");
                 return;
         }
-        
+
         if (!codec_audio_formats_compatible(pid, cid)) {
                 debug_msg("Primary (%d) and redundant (%d) not compatible\n", ppt, bpt);
                 return;
@@ -757,7 +786,7 @@ red_split_unit(u_char  ppt,        /* Primary payload type */
         cf = codec_get_format(cid);
         assert(cf != NULL);
         step = ts_map32(cf->format.sample_rate, codec_get_samples_per_frame(cid));
-        
+
         p  = b;
         pe = b + blen;
         while(p < pe) {
@@ -773,31 +802,39 @@ red_split_unit(u_char  ppt,        /* Primary payload type */
                         cu->state_len = 0;
                 }
 
-                cu->data_len = (u_int16)codec_peek_frame_size(cid, p, (u_int16)(pe - p));
+                cu->data_len = (uint16_t)codec_peek_frame_size(cid, p, (uint16_t)(pe - p));
                 cu->data     = block_alloc(cu->data_len);
                 memcpy(cu->data, p, cu->data_len);
                 p += cu->data_len;
                 md = red_media_data_create_or_get(out, playout);
                 if (md->nrep == MAX_MEDIA_UNITS) continue;
-                place_unit(md, cu);
-                playout = ts_add(playout, step);
+                if (place_unit(md, cu) == TRUE) {
+                        playout = ts_add(playout, step);
+                } else {
+                        /* unit could not be placed - destroy */
+                        if (cu->state_len) {
+                                block_free(cu->state, cu->state_len);
+                        }
+                        block_free(cu->data, cu->data_len);
+                        block_free(cu, sizeof(coded_unit));
+                }
         }
 }
 
-__inline static void
-redundancy_decoder_output(channel_unit *chu, struct s_pb *out, ts_t playout)
+static void
+redundancy_decoder_output(channel_unit *chu, struct s_pb *out, timestamp_t playout)
 {
         const codec_format_t *cf;
         codec_id_t cid;
         u_char  *hp, *dp, *de, ppt, bpt;
-        u_int32 hdr32, blen, boff;
-        ts_t ts_max_off, ts_blk_off, this_playout;
+        uint32_t hdr32, blen, boff;
+        timestamp_t ts_max_off, ts_blk_off, this_playout;
 
-        hp = dp  = chu->data + chu->data_start;
+        hp = dp = chu->data;
         de = chu->data + chu->data_len;
 
         /* move data pointer past header */
-        while (ntohl(*((u_int32*)dp)) & RED_HDR32_PAT) {
+        while (ntohl(*((uint32_t*)dp)) & RED_HDR32_PAT) {
                 dp += 4;
         }
 
@@ -814,7 +851,7 @@ redundancy_decoder_output(channel_unit *chu, struct s_pb *out, ts_t playout)
 
         /* Max offset should be in first header.  Want max offset
          * as we nobble timestamps to be:
-         *              playout + max_offset - this_offset 
+         *              playout + max_offset - this_offset
          */
 
         cid   = codec_get_by_payload(ppt);
@@ -826,7 +863,7 @@ redundancy_decoder_output(channel_unit *chu, struct s_pb *out, ts_t playout)
         cf = codec_get_format(cid);
         assert(cf != NULL);
 
-        hdr32 = ntohl(*(u_int32*)hp);
+        hdr32 = ntohl(*(uint32_t*)hp);
         ts_max_off = ts_map32(cf->format.sample_rate, RED_HDR32_GET_OFF(hdr32));
 	blen = 0;
 
@@ -843,26 +880,27 @@ redundancy_decoder_output(channel_unit *chu, struct s_pb *out, ts_t playout)
                 red_split_unit(ppt, bpt, dp, blen, this_playout, out);
                 xmemchk();
                 dp += blen;
-                hdr32 = ntohl(*(u_int32*)hp);
+                hdr32 = ntohl(*(uint32_t*)hp);
         }
-        
+
         this_playout = ts_add(playout, ts_max_off);
         hp += 1;
-        blen = (u_int32) (de - dp);
+        blen = (uint32_t) (de - dp);
         red_split_unit(ppt, ppt, dp, blen, this_playout, out);
         xmemchk();
 }
-                          
+
 int
-redundancy_decoder_decode(u_char      *state,
+redundancy_decoder_decode(u_char       *state,
                            struct s_pb *in,
                            struct s_pb *out,
-                           ts_t         now)
+                           timestamp_t  now)
 {
         struct s_pb_iterator *pi;
+        u_char               *c_get;
         channel_data         *c;
-        u_int32               clen;
-        ts_t                  cplayout;
+        uint32_t              clen;
+        timestamp_t           cplayout;
 
         pb_iterator_create(in, &pi);
         assert(pi != NULL);
@@ -870,19 +908,21 @@ redundancy_decoder_decode(u_char      *state,
         assert(state == NULL); /* No decoder state necesssary */
         UNUSED(state);
 
-        while(pb_iterator_get_at(pi, (u_char**)&c, &clen, &cplayout)) {
+        while(pb_iterator_get_at(pi, &c_get, &clen, &cplayout)) {
+                c = (channel_data*)c_get;
                 assert(c != NULL);
                 assert(clen == sizeof(channel_data));
                 if (ts_gt(cplayout, now)) {
                         break;
                 }
-                pb_iterator_detach_at(pi, (u_char**)&c, &clen, &cplayout);
+                pb_iterator_detach_at(pi, &c_get, &clen, &cplayout);
+                c = (channel_data*)c_get;
                 assert(c->nelem == 1);
                 redundancy_decoder_output(c->elem[0], out, cplayout);
                 channel_data_destroy(&c, sizeof(channel_data));
         }
-        
+
         pb_iterator_destroy(in, &pi);
         return TRUE;
 }
-        
+
