@@ -1,12 +1,16 @@
 /*
  * FILE:    codec.c
  * AUTHORS: Orion Hodson
- * 
- * Copyright (c) 1998 University College London
- * All rights reserved.
  *
+ * Copyright (c) 1998-2001 University College London
+ * All rights reserved.
  */
 
+#ifndef HIDE_SOURCE_STRINGS
+static const char cvsid[] =
+	"$Id: codec.c,v 1.75 2004/01/12 17:14:26 ucacoxh Exp $";
+#endif /* HIDE_SOURCE_STRINGS */
+
 #include "config_unix.h"
 #include "config_win32.h"
 #include "debug.h"
@@ -14,9 +18,10 @@
 /* These two just for UCL memory debugging fn's */
 #include "memory.h"
 #include "util.h"
-
+#include "audio_types.h"
 #include "codec_types.h"
 #include "codec.h"
+#include "codec_l8.h"
 #include "codec_l16.h"
 #include "codec_g711.h"
 #include "codec_g726.h"
@@ -24,10 +29,13 @@
 #include "codec_gsm.h"
 #include "codec_lpc.h"
 #include "codec_vdvi.h"
+#ifndef REMOVE_WBS_CODEC
 #include "codec_wbs.h"
+#endif
+#include "codec_g728.h"
 
-/* Codec class initialization - hey didn't c++ happen somewhere along the 
- * timeline ;-) 
+/* Codec class initialization - hey didn't c++ happen somewhere along the
+ * timeline ;-)
  */
 
 /* One time codec {con,de}struction */
@@ -35,34 +43,40 @@ typedef void    (*cx_init_f) (void);
 typedef void    (*cx_exit_f) (void);
 
 /* Codec Probing functions */
-typedef u_int16               (*cx_get_formats_count_f) (void);
-typedef const codec_format_t* (*cx_get_format_f)        (u_int16);
+typedef uint16_t               (*cx_get_formats_count_f) (void);
+typedef const codec_format_t* (*cx_get_format_f)        (uint16_t);
 
 /* Codec Encoding functions */
-typedef int     (*cx_encoder_create_f)    (u_int16 idx, u_char **state);
-typedef void    (*cx_encoder_destroy_f)   (u_int16 idx, u_char **state);
-typedef int     (*cx_encode_f)            (u_int16 idx, u_char *state, 
+typedef int     (*cx_encoder_create_f)    (uint16_t idx, u_char **state);
+typedef void    (*cx_encoder_destroy_f)   (uint16_t idx, u_char **state);
+typedef int     (*cx_encode_f)            (uint16_t idx, u_char *state,
                                            sample *in, coded_unit *out);
-typedef int     (*cx_can_encode_f)        (u_int16 idx);
+typedef int     (*cx_can_encode_f)        (uint16_t idx);
 
 /* Codec Decoding functions */
-typedef int     (*cx_decoder_create_f)    (u_int16 idx, u_char **state);
-typedef void    (*cx_decoder_destroy_f)   (u_int16 idx, u_char **state);
-typedef int     (*cx_decode_f)            (u_int16 idx, u_char *state, 
+typedef int     (*cx_decoder_create_f)    (uint16_t idx, u_char **state);
+typedef void    (*cx_decoder_destroy_f)   (uint16_t idx, u_char **state);
+typedef int     (*cx_decode_f)            (uint16_t idx, u_char *state,
                                            coded_unit *in, sample *out);
-typedef int     (*cx_can_decode_f)        (u_int16 idx);
+typedef int     (*cx_can_decode_f)        (uint16_t idx);
 
 /* For determining frame sizes of variable bit rate codecs */
-typedef int     (*cx_peek_size_f)         (u_int16 idx, u_char *data, int data_len);
+typedef int     (*cx_peek_size_f)         (uint16_t idx, u_char *data, int data_len);
 
 /* For codec domain repair schemes */
-typedef int     (*cx_repair_f)            (u_int16 idx, 
+typedef int     (*cx_repair_f)            (uint16_t idx,
                                            u_char *state,
-                                           u_int16 consec_missing,
+                                           uint16_t consec_missing,
                                            coded_unit *prev,
                                            coded_unit *missing,
                                            coded_unit *next);
 
+/* For layered codecs */
+typedef uint8_t		(*cx_can_layer_f)	  (void);
+typedef int             (*cx_get_layer_f)         (uint16_t idx, coded_unit *cu_whole, uint8_t layer,
+                                                   uint16_t *markers, coded_unit *cu_layer);
+typedef int             (*cx_combine_layer_f)     (uint16_t idx, coded_unit *cu_layer, coded_unit *cu_whole, uint8_t nelem, uint16_t *markers);
+
 typedef struct s_codec_fns {
         cx_init_f               cx_init;
         cx_exit_f               cx_exit;
@@ -78,16 +92,19 @@ typedef struct s_codec_fns {
         cx_can_decode_f         cx_can_decode;
         cx_peek_size_f          cx_peek_size;
         cx_repair_f             cx_repair;
+	cx_can_layer_f	        cx_can_layer;
+        cx_get_layer_f          cx_get_layer;
+        cx_combine_layer_f      cx_combine_layer;
 } codec_fns_t;
 
 static codec_fns_t codec_table[] = {
-        {
-                NULL, 
+	{
+                NULL,
                 NULL,
                 l16_get_formats_count,
                 l16_get_format,
                 NULL, /* No encoder setup / tear down */
-                NULL, 
+                NULL,
                 l16_encode,
                 NULL,
                 NULL,
@@ -95,6 +112,28 @@ static codec_fns_t codec_table[] = {
                 l16_decode,
                 NULL,
                 NULL,
+                NULL,
+                NULL,
+                NULL,
+                NULL
+        },
+        {
+                NULL,
+                NULL,
+                l8_get_formats_count,
+                l8_get_format,
+                NULL, /* No encoder setup / tear down */
+                NULL,
+                l8_encode,
+                NULL,
+                NULL,
+                NULL, /* No decoder setup / tear down */
+                l8_decode,
+                NULL,
+                NULL,
+                NULL,
+                NULL,
+                NULL,
                 NULL
         },
         {       g711_init,
@@ -110,8 +149,32 @@ static codec_fns_t codec_table[] = {
                 g711_decode,
                 NULL,
                 NULL,
+                NULL,
+                NULL,
+                NULL,
                 NULL
         },
+#ifdef HAVE_G728
+        {
+                NULL,
+                NULL,
+                g728_get_formats_count,
+                g728_get_format,
+                g728_encoder_create,
+                g728_encoder_destroy,
+                g728_encoder_do,
+                NULL,
+                g728_decoder_create,
+                g728_decoder_destroy,
+                g728_decoder_do,
+                NULL,
+                NULL,
+                NULL,
+                NULL,
+                NULL,
+                NULL
+        },
+#endif /* HAVE_G728 */
         {
                 NULL,
                 NULL,
@@ -126,6 +189,9 @@ static codec_fns_t codec_table[] = {
                 g726_decode,
                 NULL,
                 NULL,
+                NULL,
+                NULL,
+                NULL,
                 NULL
         },
         {
@@ -142,6 +208,9 @@ static codec_fns_t codec_table[] = {
                 dvi_decode,
                 NULL,
                 NULL,
+                NULL,
+                NULL,
+                NULL,
                 NULL
         },
         {
@@ -158,8 +227,16 @@ static codec_fns_t codec_table[] = {
                 vdvi_decoder,
                 NULL,
                 vdvi_peek_frame_size,
+                NULL,
+                NULL,
+                NULL,
                 NULL
         },
+#ifndef REMOVE_WBS_CODEC
+	/* The WB-ADPCM algorithm was developed by British Telecommunications */
+	/* plc.  Permission has been granted to use it for non-commercial     */
+	/* research and development projects.  BT retain the intellectual     */
+	/* property rights to this algorithm.                                 */
         {
                 NULL,
                 NULL,
@@ -174,8 +251,12 @@ static codec_fns_t codec_table[] = {
                 wbs_decoder,
                 NULL,
                 NULL,
-                NULL
+                NULL,
+                wbs_max_layers,
+                wbs_get_layer,
+                wbs_combine_layer
         },
+#endif
         {
                 NULL,
                 NULL,
@@ -190,7 +271,10 @@ static codec_fns_t codec_table[] = {
                 gsm_decoder,
                 NULL,
                 NULL,
-                gsm_repair
+                gsm_repair,
+                NULL,
+                NULL,
+                NULL
         },
         {
                 lpc_setup,
@@ -206,7 +290,10 @@ static codec_fns_t codec_table[] = {
                 lpc_decoder,
                 NULL,
                 NULL,
-                lpc_repair
+                lpc_repair,
+                NULL,
+                NULL,
+                NULL
         }
 };
 
@@ -216,28 +303,28 @@ static codec_fns_t codec_table[] = {
 
 /* These are used to save multiple function calls in
  * codec_get_codec_number function. */
-static u_int16 num_fmts_supported[NUM_CODEC_INTERFACES];
-static u_int16 total_fmts_supported;
+static uint16_t num_fmts_supported[NUM_CODEC_INTERFACES];
+static uint16_t total_fmts_supported;
 
 /* Codec identifier is 32 bits long. It's like an MS handle.
  * First byte is always zero.
  * Second byte is index in codec_table above plus 1.
  * Third and fourth bytes hold index for encoding format
  * used by codec.
- */ 
+ */
 
-#define CODEC_GET_IFS_INDEX(id)     (u_int16)(((id  & 0x00ff0000) >> 16) - 1)
-#define CODEC_GET_FMT_INDEX(id)     (u_int16)((id & 0x0000ffff) - 1)
+#define CODEC_GET_IFS_INDEX(id)     (uint16_t)(((id  & 0x00ff0000) >> 16) - 1)
+#define CODEC_GET_FMT_INDEX(id)     (uint16_t)((id & 0x0000ffff) - 1)
 #define CODEC_MAKE_ID(ifs,fmt)      (((ifs) + 1) << 16)|((fmt+1)&0xffff)
 
 #define CODEC_VALID_PAD(id)          (!(id & 0xff000000))
 #define CODEC_VALID_IFS(id)           (id & 0x00ff0000)
 #define CODEC_VALID_FMT(id)           (id & 0x0000ffff)
 
-__inline int
+int
 codec_id_is_valid(codec_id_t id)
 {
-        u_int32 ifs, fmt;
+        uint32_t ifs, fmt;
 
         if (codec_is_native_coding(id)) {
                 /* Native codings should be tested with
@@ -245,8 +332,8 @@ codec_id_is_valid(codec_id_t id)
                 debug_msg("Coding is invalid because it is a native coding\n");
                 return FALSE;
         }
-        
-        if (!CODEC_VALID_PAD(id) || 
+
+        if (!CODEC_VALID_PAD(id) ||
             !CODEC_VALID_IFS(id) ||
             !CODEC_VALID_FMT(id)) {
                 debug_msg("Codec id (0x%08x) invalid (pad %x, ifs %x, fmt %x)",
@@ -261,7 +348,7 @@ codec_id_is_valid(codec_id_t id)
         if (ifs >= NUM_CODEC_INTERFACES) {
                 /* Table index too large */
                 debug_msg("Codec index outside table\n");
-                return FALSE; 
+                return FALSE;
         }
 
         if (fmt >= num_fmts_supported[ifs]) {
@@ -279,8 +366,8 @@ void
 codec_init()
 {
         const codec_format_t *cf;
-        u_int32 i;
-        u_int16 j;
+        uint32_t i;
+        uint16_t j;
 
         if (total_fmts_supported == 0) {
                 for(i = 0; i < NUM_CODEC_INTERFACES; i++) {
@@ -299,12 +386,12 @@ codec_init()
         } else {
                 debug_msg("codec_init already called - ignoring.\n");
         }
-}        
+}
 
 void
 codec_exit()
 {
-        u_int32 i;
+        uint32_t i;
         if (total_fmts_supported != 0) {
                 for(i = 0; i < NUM_CODEC_INTERFACES; i++) {
                         if (codec_table[i].cx_exit) codec_table[i].cx_exit();
@@ -316,26 +403,26 @@ codec_exit()
         }
 }
 
-u_int32
+uint32_t
 codec_get_number_of_codecs()
 {
         return total_fmts_supported;
 }
 
 codec_id_t
-codec_get_codec_number(u_int32 n)
+codec_get_codec_number(uint32_t n)
 {
         codec_id_t id;
-        u_int32    ifs;
+        uint32_t    ifs;
         assert(n < total_fmts_supported);
-        
+
         for(ifs = 0; n >= num_fmts_supported[ifs]; ifs++) {
                 n = n - num_fmts_supported[ifs];
         }
 
         id = CODEC_MAKE_ID(ifs, n);
 
-        assert(codec_id_is_valid(id));        
+        assert(codec_id_is_valid(id));
 
         return id;
 }
@@ -343,7 +430,7 @@ codec_get_codec_number(u_int32 n)
 const codec_format_t*
 codec_get_format(codec_id_t id)
 {
-        u_int16 ifs, fmt;
+        uint16_t ifs, fmt;
 
         assert(codec_id_is_valid(id));
 
@@ -355,11 +442,11 @@ codec_get_format(codec_id_t id)
 int
 codec_can_encode(codec_id_t id)
 {
-        u_int16 ifs;
+        uint16_t ifs;
 
         ifs = CODEC_GET_IFS_INDEX(id);
 
-        assert(codec_id_is_valid(id));                
+        assert(codec_id_is_valid(id));
         assert(ifs < NUM_CODEC_INTERFACES);
 
         if (codec_table[ifs].cx_can_encode) {
@@ -372,18 +459,18 @@ codec_can_encode(codec_id_t id)
                         return FALSE; /* only m * 8k at the moment */
                 }
         }
-        
+
         return TRUE;
 }
 
 int
 codec_can_decode(codec_id_t id)
 {
-        u_int32 ifs;
+        uint32_t ifs;
 
         ifs = CODEC_GET_IFS_INDEX(id);
 
-        assert(codec_id_is_valid(id));        
+        assert(codec_id_is_valid(id));
         assert(ifs < NUM_CODEC_INTERFACES);
 
         if (codec_table[ifs].cx_can_decode) {
@@ -396,7 +483,7 @@ codec_can_decode(codec_id_t id)
                         return FALSE; /* Only m * 8k at moment */
                 }
         }
-        
+
         return TRUE;
 }
 
@@ -405,27 +492,27 @@ codec_audio_formats_compatible(codec_id_t id1, codec_id_t id2)
 {
         const codec_format_t *cf1, *cf2;
         int match;
-        
+
         assert(codec_id_is_valid(id1));
         assert(codec_id_is_valid(id2));
-        
+
         cf1 = codec_get_format(id1);
         cf2 = codec_get_format(id2);
 
-        match = !memcmp(&cf1->format, &cf2->format, sizeof(audio_format)); 
+        match = !memcmp(&cf1->format, &cf2->format, sizeof(audio_format));
 
         return match;
 }
 
-u_int32
+uint32_t
 codec_get_samples_per_frame(codec_id_t id)
 {
         const codec_format_t *cf;
-        u_int32 spf;
+        uint32_t spf;
 
         assert(codec_id_is_valid(id));
         cf = codec_get_format(id);
-        spf = cf->format.bytes_per_block * 8 / 
+        spf = cf->format.bytes_per_block * 8 /
                 (cf->format.channels * cf->format.bits_per_sample);
 
         return spf;
@@ -436,7 +523,7 @@ int
 codec_encoder_create(codec_id_t id, codec_state **cs)
 {
         if (codec_id_is_valid(id)) {
-                u_int16 ifs, fmt;
+                uint16_t ifs, fmt;
                 *cs = (codec_state*)block_alloc(sizeof(codec_state));
                 if (!cs) {
                         *cs = NULL;
@@ -449,7 +536,7 @@ codec_encoder_create(codec_id_t id, codec_state **cs)
                 if (codec_table[ifs].cx_encoder_create) {
                         /* Must also have a destructor */
                         assert(codec_table[ifs].cx_encoder_destroy != NULL);
-                        codec_table[ifs].cx_encoder_create(fmt, 
+                        codec_table[ifs].cx_encoder_create(fmt,
                                                            &(*cs)->state);
                 }
                 return TRUE;
@@ -467,12 +554,12 @@ codec_encoder_destroy(codec_state **cs)
         assert(*cs != NULL);
         id = (*cs)->id;
         if (codec_id_is_valid(id)) {
-                u_int16 ifs, fmt;
+                uint16_t ifs, fmt;
                 ifs = CODEC_GET_IFS_INDEX(id);
                 fmt = CODEC_GET_FMT_INDEX(id);
                 if (codec_table[ifs].cx_encoder_destroy) {
                         /* Must also have a destructor */
-                        codec_table[ifs].cx_encoder_destroy(fmt, 
+                        codec_table[ifs].cx_encoder_destroy(fmt,
                                                                &(*cs)->state);
                 }
                 block_free(*cs, sizeof(codec_state));
@@ -488,7 +575,7 @@ codec_encode(codec_state *cs,
              coded_unit  *in_native,
              coded_unit  *cu)
 {
-        u_int16    ifs, fmt;
+        uint16_t    ifs, fmt;
         int        success;
 
         assert(cs        != NULL);
@@ -519,7 +606,7 @@ int
 codec_decoder_create(codec_id_t id, codec_state **cs)
 {
         if (codec_id_is_valid(id)) {
-                u_int16 ifs, fmt;
+                uint16_t ifs, fmt;
                 *cs = (codec_state*)block_alloc(sizeof(codec_state));
                 if (!cs) {
                         *cs = NULL;
@@ -532,7 +619,7 @@ codec_decoder_create(codec_id_t id, codec_state **cs)
                 if (codec_table[ifs].cx_decoder_create) {
                         /* Must also have a destructor */
                         assert(codec_table[ifs].cx_decoder_destroy != NULL);
-                        codec_table[ifs].cx_decoder_create(fmt, 
+                        codec_table[ifs].cx_decoder_create(fmt,
                                                                &(*cs)->state);
                 }
                 return TRUE;
@@ -550,12 +637,12 @@ codec_decoder_destroy(codec_state **cs)
         assert(*cs != NULL);
         id = (*cs)->id;
         if (codec_id_is_valid(id)) {
-                u_int16 ifs, fmt;
+                uint16_t ifs, fmt;
                 ifs = CODEC_GET_IFS_INDEX(id);
                 fmt = CODEC_GET_FMT_INDEX(id);
                 if (codec_table[ifs].cx_decoder_destroy) {
                         /* Must also have a destructor */
-                        codec_table[ifs].cx_decoder_destroy(fmt, 
+                        codec_table[ifs].cx_decoder_destroy(fmt,
                                                                &(*cs)->state);
                 }
                 block_free(*cs, sizeof(codec_state));
@@ -573,13 +660,13 @@ codec_decode(codec_state *cs,
 {
         const codec_format_t *cf;
         codec_id_t           id;
-        u_int16              ifs, fmt, rate, channels;
+        uint16_t              ifs, fmt, rate, channels;
         int                  success;
 
         assert(cs  != NULL);
         assert(out != NULL);
         assert(in  != NULL);
-        
+
         id = cs->id;
         assert(in->id == cs->id);
         assert(codec_is_native_coding(in->id) == FALSE);
@@ -591,24 +678,24 @@ codec_decode(codec_state *cs,
         cf = codec_get_format(id);
         assert(out->state == NULL);
         assert(out->data  == NULL);
-        rate     = (u_int16)cf->format.sample_rate;
-        channels = (u_int16)cf->format.channels;
+        rate     = (uint32_t)cf->format.sample_rate;
+        channels = (uint16_t)cf->format.channels;
         out->id       = codec_get_native_coding(rate, channels);
         out->data_len = cf->format.bytes_per_block;
         out->data     = (u_char*)block_alloc(out->data_len);
 
         /* Decode */
         xmemchk();
-        success =  codec_table[ifs].cx_decode(fmt, cs->state, in, (sample*)out->data);
+        success = codec_table[ifs].cx_decode(fmt, cs->state, in, (sample*)out->data);
         xmemchk();
 
         return success;
 }
 
-__inline int
-codec_decoder_can_repair (codec_id_t id) 
+int
+codec_decoder_can_repair (codec_id_t id)
 {
-        u_int16 ifs;
+        uint16_t ifs;
         assert(codec_id_is_valid(id));
         ifs = CODEC_GET_IFS_INDEX(id);
         if (codec_table[ifs].cx_repair) {
@@ -619,13 +706,13 @@ codec_decoder_can_repair (codec_id_t id)
 }
 
 int
-codec_decoder_repair(codec_id_t id, codec_state *cs, 
-                     u_int16 consec_missing, 
-                     coded_unit *prev, 
-                     coded_unit *miss, 
+codec_decoder_repair(codec_id_t id, codec_state *cs,
+                     uint16_t consec_missing,
+                     coded_unit *prev,
+                     coded_unit *miss,
                      coded_unit *next)
 {
-        u_int16    ifs, fmt;
+        uint16_t    ifs, fmt;
 
         assert(codec_id_is_valid(id));
         assert(id == cs->id);
@@ -640,21 +727,21 @@ codec_decoder_repair(codec_id_t id, codec_state *cs,
         miss->id = prev->id;
 
         assert(codec_table[ifs].cx_repair != NULL);
-        return codec_table[ifs].cx_repair(fmt, 
-                                          cs->state, 
+        return codec_table[ifs].cx_repair(fmt,
+                                          cs->state,
                                           consec_missing,
                                           prev, miss, next);
 }
 
-u_int32 
-codec_peek_frame_size(codec_id_t id, u_char *data, u_int16 blk_len)
+uint32_t
+codec_peek_frame_size(codec_id_t id, u_char *data, uint16_t blk_len)
 {
-        u_int16    ifs, fmt;
+        uint16_t    ifs, fmt;
 
         assert(codec_id_is_valid(id));
 
         ifs = CODEC_GET_IFS_INDEX(id);
-        fmt = CODEC_GET_FMT_INDEX(id);        
+        fmt = CODEC_GET_FMT_INDEX(id);
 
         if (codec_table[ifs].cx_peek_size) {
                 return codec_table[ifs].cx_peek_size(fmt, data, (int)blk_len);
@@ -690,8 +777,8 @@ payload_is_valid(u_char pt)
         return FALSE;
 }
 
-/* RTP Mapping interface - 
- * 2 maps one from payload to codec id and the other from codec id to 
+/* RTP Mapping interface -
+ * 2 maps one from payload to codec id and the other from codec id to
  * to payload.
  */
 
@@ -702,7 +789,7 @@ static u_char    *codec_map[NUM_CODEC_INTERFACES];
 static void
 codec_map_init()
 {
-        u_int16 i,j;
+        uint16_t i,j;
         const codec_format_t *fmt;
 
         memset(payload_map, 0, NUM_PAYLOADS * sizeof(codec_id_t));
@@ -722,7 +809,7 @@ codec_map_init()
 static void
 codec_map_exit()
 {
-        u_int32 i;
+        uint32_t i;
         for(i = 0; i < NUM_CODEC_INTERFACES; i++) {
                 xfree(codec_map[i]);
         }
@@ -763,10 +850,10 @@ codec_get_payload(codec_id_t id)
         return CODEC_PAYLOAD_DYNAMIC;
 }
 
-int 
+int
 codec_unmap_payload(codec_id_t id, u_char pt)
 {
-        if (payload_is_valid(pt) && 
+        if (payload_is_valid(pt) &&
             codec_id_is_valid(id) &&
             payload_map[pt] == id) {
                 payload_map[pt] = 0;
@@ -785,7 +872,7 @@ codec_get_by_payload (u_char pt)
                 if (payload_map[pt] == 0) {
                         debug_msg("No codec for payload %d\n", pt);
                 }
-#endif       
+#endif
                 return payload_map[pt];
         } else {
                 debug_msg("codec_get_by_payload - invalid payload (%d)\n", pt);
@@ -794,12 +881,12 @@ codec_get_by_payload (u_char pt)
 }
 
 /* For compatibility only */
-codec_id_t 
-codec_get_first_mapped_with(u_int16 sample_rate, u_int16 channels)
+codec_id_t
+codec_get_first_mapped_with(uint32_t sample_rate, uint16_t channels)
 {
         const codec_format_t *cf;
         int pt;
-        
+
         for(pt = 0; pt < NUM_PAYLOADS; pt++) {
                 if (payload_map[pt]) {
                         cf = codec_get_format(payload_map[pt]);
@@ -815,11 +902,11 @@ codec_get_first_mapped_with(u_int16 sample_rate, u_int16 channels)
 }
 
 
-codec_id_t 
+codec_id_t
 codec_get_by_name(const char *name)
 {
         const codec_format_t *cf;
-        u_int16 ifs, fmt;
+        uint16_t ifs, fmt;
 
         for(ifs = 0; ifs < NUM_CODEC_INTERFACES; ifs++) {
                 for(fmt = 0; fmt < num_fmts_supported[ifs]; fmt++) {
@@ -835,22 +922,22 @@ codec_get_by_name(const char *name)
 
 
 codec_id_t
-codec_get_matching(const char *short_name, u_int16 freq, u_int16 channels)
+codec_get_matching(const char *short_name, uint32_t freq, uint16_t channels)
 {
         /* This has been changed to try really hard to find a matching codec.
-         * The reason is that it's now called as part of the command-line      
-         * parsing, and so has to cope with user entered codec names. Also, it 
-         * should recognise the names sdr gives the codecs, for compatibility 
-         * with rat-v3.0.                                                [csp] 
+         * The reason is that it's now called as part of the command-line
+         * parsing, and so has to cope with user entered codec names. Also, it
+         * should recognise the names sdr gives the codecs, for compatibility
+         * with rat-v3.0.                                                [csp]
          */
 
         /* This is not quite as inefficient as it looks, since stage 1 will
-         * almost always find a match.                                     
+         * almost always find a match.
          */
 
         const codec_format_t  *cf = NULL;
-        codec_id_t             cid = 0; 
-        u_int32                i, codecs;
+        codec_id_t             cid = 0;
+        uint32_t                i, codecs;
         char                  *long_name;
 
         /* Stage 1: Try the designated short names... */
@@ -858,8 +945,8 @@ codec_get_matching(const char *short_name, u_int16 freq, u_int16 channels)
         for(i = 0; i < codecs; i++) {
                 cid = codec_get_codec_number(i);
                 cf  = codec_get_format(cid);
-                if (cf->format.sample_rate == freq  && 
-                    cf->format.channels == channels && 
+                if (cf->format.sample_rate == freq  &&
+                    cf->format.channels == channels &&
                     !strcasecmp(short_name, cf->short_name)) {
                         return cid;
                 }
@@ -869,22 +956,26 @@ codec_get_matching(const char *short_name, u_int16 freq, u_int16 channels)
         long_name = (char *) xmalloc(strlen(short_name) + 12);
         sprintf(long_name, "%s-%dK-%s", short_name, freq/1000, channels==1?"MONO":"STEREO");
         for(i = 0; i < codecs; i++) {
-                if (cf->format.sample_rate == freq  && 
-                    cf->format.channels == channels && 
+                cid = codec_get_codec_number(i);
+                cf  = codec_get_format(cid);
+                if (cf->format.sample_rate == freq  &&
+                    cf->format.channels == channels &&
                     !strcasecmp(long_name, cf->long_name)) {
                         xfree(long_name);
                         return cid;
                 }
         }
 
-        /* Stage 3: Nasty hack... PCM->PCMU for compatibility with sdr 
-         * and old rat versions 
+        /* Stage 3: Nasty hack... PCM->PCMU for compatibility with sdr
+         * and old rat versions
          */
-        if (strcasecmp(short_name, "pcm") == 0) {
+        if (strncasecmp(short_name, "pcm", 3) == 0) {
                 sprintf(long_name, "PCMU-%dK-%s", freq/1000, channels==1?"MONO":"STEREO");
                 for(i = 0; i < codecs; i++) {
-                        if (cf->format.sample_rate == freq  && 
-                            cf->format.channels == channels && 
+                        cid = codec_get_codec_number(i);
+                        cf  = codec_get_format(cid);
+                        if (cf->format.sample_rate == freq  &&
+                            cf->format.channels == channels &&
                             !strcasecmp(long_name, cf->long_name)) {
                                 xfree(long_name);
                                 return cid;
@@ -898,50 +989,111 @@ codec_get_matching(const char *short_name, u_int16 freq, u_int16 channels)
         return 0;
 }
 
-/* These three functions are a hack so we can have
- * encode and decode functions take coded_units as
- * input and output.  This makes paths cleaner
- * since we don't have two data types for coded and
- * raw units.
- */
 
-__inline codec_id_t 
-codec_get_native_coding(u_int16 sample_rate, u_int16 channels)
+/* These constants are what are supported as native codings */
+
+static uint16_t sampling_rates[] = {8000, 11025, 16000, 22050, 32000, 44100, 48000};
+static uint16_t num_sampling_rates = sizeof(sampling_rates)/sizeof(sampling_rates[0]);
+static uint16_t max_channels = 2;
+
+/* The following three functions are a hack so we can have encode and
+ * decode functions take coded_units as input and output.  This makes
+ * paths cleaner since we don't have two data types for coded and raw
+ * units.  */
+
+codec_id_t
+codec_get_native_coding(uint32_t sample_rate, uint16_t channels)
 {
         codec_id_t cid;
+        uint32_t i, index;
 
-        assert(sample_rate % 8000 == 0 && sample_rate <= 48000);
-        assert(channels == 1 || channels == 2);
-
-        channels    = channels - 1;
-        sample_rate = sample_rate / 4000 - 2;
+        for (i = 0; i < num_sampling_rates; i++) {
+                if (sampling_rates[i] == sample_rate) {
+                        break;
+                }
+        }
+        assert(i != num_sampling_rates);
+        assert(channels <= max_channels);
+        index = i * max_channels + (channels - 1);
         /* There is no codec corresponding to this but make it
          * have right form we set it interfaces to the number
          * of interfaces, i.e. one more than is legal.
          */
-        cid = CODEC_MAKE_ID(NUM_CODEC_INTERFACES, (sample_rate + channels));
+        cid = CODEC_MAKE_ID(NUM_CODEC_INTERFACES, index);
         return cid;
 }
 
-__inline int
+int
 codec_is_native_coding(codec_id_t cid)
 {
         return (CODEC_GET_IFS_INDEX(cid) == NUM_CODEC_INTERFACES &&
-                CODEC_GET_FMT_INDEX(cid) < 12);
+                CODEC_GET_FMT_INDEX(cid) < num_sampling_rates * max_channels);
 }
 
-__inline int 
-codec_get_native_info(codec_id_t cid, 
-                      u_int16   *p_rate, 
-                      u_int16   *p_channels)
+int
+codec_get_native_info(codec_id_t cid,
+                      uint32_t   *p_rate,
+                      uint16_t   *p_channels)
 {
+        uint32_t i, c, index;
+
         if (codec_is_native_coding(cid)) {
-                u_int32 f = CODEC_GET_FMT_INDEX(cid);
-                *p_rate   = (u_int16)(f - (f&0x01)+2) * 4000;
-                assert(*p_rate % 8000 == 0 && *p_rate <= 48000);
-                *p_channels = (u_int16)(f&0x01) + 1;
-                assert(*p_channels == 1 || *p_channels == 2);
+                index = CODEC_GET_FMT_INDEX(cid);
+                /* Calculate and verify index in table of acceptable rates */
+                i = index / max_channels;
+                if (p_rate != NULL) {
+                        *p_rate = sampling_rates[i];
+                }
+                /* Calculate and verify number of channels */
+                c = (index % max_channels) + 1;
+                if (p_channels != NULL) {
+                        *p_channels = (uint16_t)c;
+                }
                 return TRUE;
         }
         return FALSE;
 }
+
+/* Layered codecs ***********************************************************/
+uint8_t
+codec_can_layer(codec_id_t id)
+{
+        uint16_t ifs;
+
+        ifs = CODEC_GET_IFS_INDEX(id);
+
+        assert(codec_id_is_valid(id));
+        assert(ifs < NUM_CODEC_INTERFACES);
+
+        if (codec_table[ifs].cx_can_layer) {
+                return codec_table[ifs].cx_can_layer();
+        }
+        return 1;
+
+}
+
+int
+codec_get_layer(codec_id_t id, coded_unit *cu_whole, uint8_t layer, uint16_t *markers, coded_unit *cu_layer)
+{
+        uint16_t ifs, fmt;
+
+        ifs = CODEC_GET_IFS_INDEX(id);
+        fmt = CODEC_GET_FMT_INDEX(id);
+
+        if (codec_table[ifs].cx_get_layer) {
+                return codec_table[ifs].cx_get_layer(fmt, cu_whole, layer, markers, cu_layer);
+        }
+        return FALSE;
+}
+
+int
+codec_combine_layer (codec_id_t id, coded_unit *cu_layer, coded_unit *cu_whole, uint8_t nelem, uint16_t *markers)
+{
+        uint16_t ifs, fmt;
+
+        ifs = CODEC_GET_IFS_INDEX(id);
+        fmt = CODEC_GET_FMT_INDEX(id);
+
+        assert(codec_table[ifs].cx_combine_layer);
+        return codec_table[ifs].cx_combine_layer(fmt, cu_layer, cu_whole, nelem, markers);
+}
