@@ -2,20 +2,16 @@
  * FILE:    render_3D.h
  * PROGRAM: RAT
  * AUTHORS: Marcus Iken
- * 
- * $Revision: 1.12 $
- * $Date: 1999/03/24 23:02:45 $
- * 
- * Copyright (c) 1998 University College London
+ *
+ * Copyright (c) 1998-2001 University College London
  * All rights reserved.
  *
+ * $Id: render_3D.h,v 1.17 2004/01/12 17:14:41 ucacoxh Exp $
  */
 
 #ifndef __RENDER_3D_H__
 #define __RENDER_3D_H__
 
-#include <math.h>
-#include "codec_types.h"
 struct s_render_3D_dbentry;
 
 int   render_3D_filter_get_count(void);
@@ -30,19 +26,19 @@ struct s_render_3D_dbentry*
       render_3D_init (int sampling_rate);
 
 void  render_3D_free (struct s_render_3D_dbentry **data);
-void  render_3D      (struct s_render_3D_dbentry *data, 
-                      coded_unit *in_native, 
+void  render_3D      (struct s_render_3D_dbentry *data,
+                      coded_unit *in_native,
                       coded_unit *out_native);
 
-void render_3D_set_parameters (struct s_render_3D_dbentry *p_3D_data, 
-                               int sampling_rate, 
-                               int azimuth, 
-                               int filter_number, 
+void render_3D_set_parameters (struct s_render_3D_dbentry *p_3D_data,
+                               int sampling_rate,
+                               int azimuth,
+                               int filter_number,
                                int length);
 
-void render_3D_get_parameters (struct s_render_3D_dbentry *p_3D_data, 
-                               int *azimuth, 
-                               int *filter_type, 
+void render_3D_get_parameters (struct s_render_3D_dbentry *p_3D_data,
+                               int *azimuth,
+                               int *filter_type,
                                int *filter_length);
 
 #endif /* __RENDER_3D_H__ */
