@@ -1,19 +1,20 @@
 /*
  * FILE:    usleep.c
  * PROGRAM: RAT
- * AUTHORS: Colin Perkins 
- * 
+ * AUTHORS: Colin Perkins
+ *
  * Some platforms, notably Win32 and Irix 5.3 don't have a usleep
  * system call. This file provides a workaround.
  *
- * $Revision: 1.1 $
- * $Date: 1999/05/28 12:15:26 $
- * 
- * Copyright (c) 1999 University College London
+ * Copyright (c) 1999-2001 University College London
  * All rights reserved.
- *
  */
 
+#ifndef HIDE_SOURCE_STRINGS
+static const char cvsid[] =
+	"$Id: usleep.c,v 1.7 2004/01/12 17:14:49 ucacoxh Exp $";
+#endif /* HIDE_SOURCE_STRINGS */
+
 #include "config_unix.h"
 #include "config_win32.h"
 
@@ -28,7 +29,9 @@ int usleep(unsigned int usec)
 }
 #endif
 
-#ifdef IRIX
+/* On Irix 6.5.4m this function isn't needed.... we should probably have */
+/* a check in the configure script for usleep().                         */
+#ifdef NEED_USLEEP
 int usleep(unsigned int usec)
 {
 	struct timespec sleeptime,t2;
